<?PHP
/*
	请求接收程序
	-------------------

	作用：接收主控会员身份认证及财务相关操作的请求，并返回值或状态


	@更新历史
	+ 2012-5-30 增加权限限制功能


帮助
----------


A 主控请求方式
	使用POST请求接口地址如：http://domain.com/api/api.php
	数据：action=操作类型&sign=流水号&其它参数

	sign格式：YYYYMMDDHHIISS######（20位，前8位必须为年月日；之后的值保证每次不同即可，建议为时分秒加6位随机数）

	正确时返回<return>0</return><##>***</##>（其它标签）

	错误时返回<return>1至9</return><msg>错误信息</msg><##>***</##> （其它标签）


B 操作类型
 
  [1] action=checkConnect （检查接口是否能正常连接）
	  参数
	 sign    =流水号   （每次请求必须不一样，使用过的流水号不能再使用）
	 encode  =MD5验证码，值为 md5(API密码 + action + 流水号)
	 返回
		<return>0</return>

  [2] action=checkUser （验证会员登录）
	 参数：
	 username=登录账号
	 sign    =流水号   （每次请求必须不一样，使用过的流水号不能再使用）
	 encode  =MD5验证码，值为 md5(API密码 + action + 流水号 + 登录账号)
	 check   =用户密码

	 返回
		<return>0</return><userid>123</userid><username>登录账号</username><discount>折扣率</discount><userlimit>用户操作权限</userlimit>

userlimit

限制值说明： 

A 禁止收费新购买  B 禁止收费续期 C 禁止收费升级

F 禁止免费申请 G 禁止免费续期，H 禁止免费升级 T 禁止申请试用

V 通过身份认证

比如A 禁止购买收费


  [3] action=checkLogin （验证自动登录）
	 userid  =会员编号
	 sign    =流水号   （每次请求必须不一样，使用过的流水号不能再使用）
	 encode  =MD5验证码，值为 md5(API密码 + action + 流水号 + 会员编号)
	 check   =密码加密字符串，值为md5(API密码 + 流水号 + 临时密码)

	 说明：如果 signtype=1，通过验证后需要将数据表中的附加密码(比如表site_user的user_signkey)清空（保证只能登录一次）

	 返回
		<return>0</return><userid>123</userid><username>登录账号</username><discount>折扣率</discount><userlimit>操作权限</userlimit>

  [4] action=queryFund （查询会员余额）
	 参数
	 userid  =会员编号
	 sign    =流水号   （每次请求必须不一样，使用过的流水号不能再使用）
	 encode  =MD5验证码，值为 md5(API密码 +action + 流水号 + 用户编号) 

	 返回 <return>0</return><fund>余额</fund>

  [5] action=addFund  （写入财务记录）
	 userid  =会员编号
	 sign    =流水号   （每次请求必须不一样，使用过的流水号不能再使用）
	 fund    =金额
	 fundtype=类型 1 购买 2 续费 3 升级
	 orderid =订单号
	 encode  =MD5验证码，值为 md5(API密码 +action + 流水号 + 用户编号 + 金额 + 类型 + 订单号) 
	 remark  =备注

	 返回 <return>0</return><fund>扣款后的余额</font>

  [6] action=addNotice （写入通知）
	 userid  =会员编号
	 sign    =流水号   （每次请求必须不一样，使用过的流水号不能再使用）
	 orderid =订单号(相关联的订单号)
	 sentmail=是否同时发送邮件通知（0不发送邮件通知; 1 同时发送邮件通知）
	 title   =标题
	 msg     =内容
	 encode  =MD5验证码，值为 md5(API密码 +action + 流水号 + 用户编号 + 订单号 + 标题 + 内容) 

	 返回 <return>0</return>

C 错误代码

	<return>1</return><msg>密码验证失败</msg>
	<return>2</return><msg>sign错误或重复</msg>
	<return>3</return><msg>账号不存在</msg>
	<return>4</return><msg>余额不足</msg>
	<return>9</return><msg>非1-4的错误</msg>

*/


class API
{
	var $DB;
	var $Tbl;
	var $dir;
	var $vkey;


	function API($DB,$tblArr,$log_dir)
	{
		$this->DB=$DB;
		$this->Tbl=$tblArr;
		$this->dir=$logdir;
	}

	function response()
	{
		$data=$this->getpost();
		$log_dir=$this->dir;
		if (!preg_match('/^[1-9][0-9a-f]{19,31}$/i',$data['sign']))
		{
			echo '<return>9</return><msg>sign错误！</msg>';
			exit;
		}

		$t1=date('Ymd',time()-24*3600);
		$t2=date('Ymd',time()+24*3600);
		$t0=substr($data['sign'],0,8);

		//检查sign
		if ($t0<$t1 or $t0>$t2)
		{
			echo '<return>2</return><msg>sign错误！</msg>';
			exit;
		}

		if (strlen($this->vkey)<16)
		{
			echo '<return>1</return><msg>接口密码格式错误</msg>';
			exit;
		}

		if (strlen($data['encode'])<20)
		{
			echo '<return>1</return><msg>密码验证失败</msg>';
			exit;
		}

		$signfile='';
		//使用文件记录sign是否使用过
		if (file_exists($log_dir))
		{
			$dir=$log_dir.$t0.'/';
			if (!file_exists($dir))
			{
				@mkdir($dir);
			}

			$signfile=$dir.$data['sign'].'.log';
			if (file_exists($signfile))
			{
				echo '<return>2</return><msg>sign已经使用过</msg>';
				exit;
			}
		}

		if ($data['action']=='checkConnect')
		{
			$this->checkConnect($data);
		}
		else if ($data['action']=='checkUser')
		{
			$this->checkUser($data);
		}
		else if ($data['action']=='checkLogin')
		{
			$this->checkLogin($data);
		}
		else if ($data['action']=='queryFund')
		{
			$this->queryFund($data);
		}
		else if ($data['action']=='addFund')
		{
			$this->addFund($data);
		}
		else if ($data['action']=='addNotice')
		{
			$this->addNotice($data);
		}
		else
		{
			echo '<return>9</return><msg>请求类型错误！</msg>';
		}

		if (!empty($signfile))
		{
			$this->writeFile($signfile,'w+',$data['action'].date('Y-m-d H:i:s'));
		}
	}

	function checkConnect($data)
	{
		if ($data['encode']!=md5($this->vkey.$data['action'].$data['sign']))
		{
			echo '<return>1</return><msg>无法正常连接！</msg>';
			return 0;
		}
		echo '<return>0</return>';
		return 1;
	}

	//验证会员账号登录
	function checkUser($data)
	{
		if ($data['encode']!=md5($this->vkey.$data['action'].$data['sign'].$data['username']))
		{
			echo '<return>1</return><msg>登录验证失败！</msg>';
			return 0;
		}

		$data['username']=trim($data['username']);
		if (empty($data['username']))
		{
			echo '<return>9</return><msg>会员账号不能为空！</msg>';
			return 0;
		}

		$sql="select * from ".$this->Tbl['user']." where user_name=".$this->DB->getsql($data['username']);
		$uinfo=$this->DB->query($sql,1);
		if(empty($uinfo)){
			echo '<return>3</return><msg>账号不存在</msg>';
			return 0;
		}
		else if($uinfo['user_stat']>0){
			echo '<return>3</return><msg>账号暂时无法使用！</msg>';
			return 0;
		}

		if ($uinfo['user_errdate']==date('Y-m-d') and $uinfo['user_errtimes']>=10)
		{
			echo '<return>9</return><msg>当前账号输入错误密码超过10次,本日不能再登录，请次日再试</msg>';
			return 0;
		}

		$api_encode=md5($this->vkey.$data['sign'].$uinfo['user_password']);
		if($data['check']!=$api_encode){
			$this->DB->query("update ".$this->Tbl['user']." set user_errtimes=if(user_errdate=CURDATE(),user_errtimes+1,1),user_errdate=CURDATE() where user_id=".intval($uinfo['user_id']));
			echo '<return>1</return><msg>身份验证失败！</msg>';
			return 0;
		}

		//通过验证，执行相应操作
		$userlimit=$uinfo['user_limit'];
		if($uinfo['user_checktype']>20010101){
			$userlimit=$userlimit.'V';
		}

		$sql="select sum(fund) as fund from ".$this->Tbl['fund']." where fund_stat=0 and user_id=".intval($uinfo['user_id']);
		$res = $this->DB->query($sql,1);
		$funds=0+doubleval($res['fund']);
		echo '<return>0</return><userid>'.intval($uinfo['user_id']).'</userid><username>'.$data['username'].'</username><discount>1</discount><userlimit>'.$userlimit.'</userlimit><fund>'.$funds.'</fund>';

	}

	//验证自动登录
	function checkLogin($data)
	{
		if ($data['encode']!=md5($this->vkey.$data['action'].$data['sign'].$data['userid']))
		{
			echo '<return>1</return><msg>登录验证失败!</msg>';
			return 0;
		}

		$data['userid']=intval($data['userid']);
		if (empty($data['userid']))
		{
			echo '<return>9</return><msg>会员编号不能为空！</msg>';
			return 0;
		}

		$sql="select * from ".$this->Tbl['user']." where user_id=".$this->DB->getsql($data['userid']);
		$uinfo=$this->DB->query($sql,1);
		if(empty($uinfo)){
			echo '<return>3</return><msg>账号不存在！</msg>';
			return 0;
		}
		else if($uinfo['user_stat']>0){
			echo '<return>3</return><msg>账号暂时无法使用！</msg>';
			return 0;
		}

		if (empty($uinfo['user_signkey']))
		{
			echo '<return>9</return><msg>密码验证失败！</msg>';
			return 0;
		}

		if ('_'.substr($uinfo['user_signkey'],0,14)<date('_YmdHis',time()-15))//15秒内有效
		{
			echo '<return>9</return><msg>验证超时！</msg>';
			return 0;
		}

		$api_encode=md5($this->vkey.$data['sign'].$uinfo['user_signkey']);
		if($data['check']!=$api_encode){
			echo '<return>1</return><msg>身份验证失败！</msg>';
			return 0;
		}

		$userlimit=$uinfo['user_limit'];
		if($uinfo['user_checktype']>20010101){
			$userlimit=$userlimit.'V';
		}

		//通过验证，执行相应操作
		$ret=$this->DB->query("update ".$this->Tbl['user']." set user_signkey='' where user_id=".intval($data['userid']));

		$sql="select sum(fund) as fund from ".$this->Tbl['fund']." where fund_stat=0 and user_id=".intval($data['userid']);
		$res = $this->DB->query($sql,1);
		$funds=0+doubleval($res['fund']);
		echo '<return>0</return><userid>'.intval($uinfo['user_id']).'</userid><username>'.$uinfo['user_name'].'</username><discount>1</discount><userlimit>'.$userlimit.'</userlimit><fund>'.$funds.'</fund>';

	}

	//查询会员余额
	function queryFund($data)
	{
		$userid=intval($data['userid']);
		$api_encode=md5($this->vkey.$data['action'].$data['sign'].$userid);
		if($data['encode']!=$api_encode){
			echo '<return>1</return><msg>登录验证失败！</msg>';
			return 0;
		}

		if($userid<=0){
			echo '<return>9</return><msg>会员编号不能为空！</msg>';
			return 0;
		}

		$uinfo=$this->DB->query("select * from ".$this->Tbl['user']." where user_stat<4 and user_id=".intval($userid),1);
		if (empty($uinfo))
		{
			echo '<return>3</return><msg>会员编号'.$userid.'不存在</msg>';
			return 0;
		}

		$userlimit=$uinfo['user_limit'];
		if($uinfo['user_checktype']>20010101){
			$userlimit=$userlimit.'V';
		}

		//通过验证，执行相应操作
		$sql="select sum(fund) as fund from ".$this->Tbl['fund']." where fund_stat=0 and user_id=".intval($userid);
		$funds = $this->DB->query($sql,1);
		echo '<return>0</return><fund>'.(0+$funds['fund']).'</fund><userlimit>'.$userlimit.'</userlimit>';
	}


	//写入财务记录
	function addFund($data)
	{
		$add_cmd_str=intval($data['userid']);
		$add_cmd_str.=doubleval($data['fund']);
		$add_cmd_str.=intval($data['fundtype']);
		$add_cmd_str.=intval($data['orderid']);
		$api_encode=md5($this->vkey.$data['action'].$data['sign'].$add_cmd_str);
		if($data['encode']!=$api_encode){
			echo '<return>1</return><msg>登录验证失败！</msg>';
			return 0;
		}

		$userid=intval($data['userid']);
		if($userid<=0){
			echo '<return>9</return><msg>会员编号不能为空！</msg>';
			return 0;
		}


		$users=$this->DB->query("select * from ".$this->Tbl['user']." where  user_stat<4 and user_id=".intval($userid),1);
		if (empty($users))
		{
			echo '<return>3</return><msg>会员编号'.$userid.'不存在</msg>';
			return 0;
		}

		//通过验证，执行相应操作
		$fund=doubleval($data['fund']);
		$sql="select sum(fund) as fund from ".$this->Tbl['fund']." where fund_stat=0 and user_id=".intval($userid);
		$res = $this->DB->query($sql,1);
		$funds=0+doubleval($res['fund']);

		if ($data['fundtype']==1) $fundtype=21;
		else if ($data['fundtype']==2) $fundtype=22;
		else if ($data['fundtype']==3) $fundtype=23;

		/*
		if ($fund>0 and $fund>$funds)
		{
			echo '<return>4</return><msg>会员账号余额不足！</msg>';
			return 0;
		}
		*/

		//余额检查放到客户端进行，扣款前使用queryFund。因为必须先操作再扣费，操作不成功不扣费
		$sql="insert into ".$this->Tbl['fund']." set fund_date=CURDATE(),add_time=NOW()";
		$sql=$sql.",fund_type=$fundtype";
		$sql=$sql.",user_id=".$userid;
		$sql=$sql.",fund_orderid=".intval($data['orderid']);
		$sql=$sql.",fund=".(0-$fund);
		$sql=$sql.",fund_remark=".$this->DB->getsql($data['remark']);
		$res=$this->DB->query($sql);
		if($res){
			echo '<return>0</return><fund>'.($funds-$fund).'</fund>';
		}else{
			echo '<return>9</return><fund>'.$funds.'</fund><msg>扣款失败！</msg>';
		}

	}

	//写入通知
	function addNotice($data)
	{
		$add_cmd_str=intval($data['userid']);
		$add_cmd_str.=intval($data['orderid']);
		$add_cmd_str.=$data['title'];
		$add_cmd_str.=$data['msg'];
		$api_encode=md5($this->vkey.$data['action'].$data['sign'].$add_cmd_str);
		if($data['encode']!=$api_encode){
			echo '<return>1</return><msg>登录验证失败！</msg>';
			return 0;
		}

		$userid=intval($data['userid']);
		if($userid<=0){
			echo '<return>9</return><msg>会员编号不能为空！</msg>';
			return 0;
		}
		if (empty($data['title']))
		{
			echo '<return>9</return><msg>通知标题不能为空</msg>';
			return 0;
		}
		if (empty($data['msg']))
		{
			echo '<return>9</return><msg>通知内容不能为空</msg>';
			return 0;
		}

		$users=$this->DB->query("select * from ".$this->Tbl['user']." where  user_stat<4 and user_id=".intval($userid),1);
		if (empty($users))
		{
			echo '<return>3</return><msg>会员编号'.$userid.'不存在</msg>';
			return 0;
		}

		//通过验证，执行相应操作
		$sentmail=intval($data['sentmail']);
		//发送邮件通知
		if ($sentmail and !empty($users['user_name']))
		{
			$errMsg=mailto($users['user_name'],$data['title'],$data['msg'],'text','UTF-8');
			$sql="insert into ".$this->Tbl['mail']." set ";
			if(empty($errMsg))
			{
				$sql.="mail_stat=0";
			}
			else
			{
				$sql.="mail_stat=1";
				$sql.=",mail_errmsg=".$this->DB->getsql($errMsg);
			}
			$sql.=",mail_senttime=NOW()";
			$sql.=",user_id=".intval($userid);
			$sql.=",mail_to=".$this->DB->getsql($users['user_name']);
			$sql.=",mail_title=".$this->DB->getsql($data['title']);
			$sql.=",mail_cont=".$this->DB->getsql($data['msg']);
			if($this->DB->query($sql)){
				
			}else{
				echo '<return>3</return><msg>邮件保存失败！</msg>';
				return 0;
			}

		}

		//写入站内通知（如果需要的话）
		//......

		echo '<return>0</return>';

	}


	function signKey($userid)
	{
		$sign=date('YmdHis').substr(microtime(),2,4).rand(0,9).rand(0,9).rand(0,9).rand(0,9).rand(0,9).rand(0,9);
		$sign=$sign.rand(0,9).rand(0,9).rand(0,9).rand(0,9);
		$sign=substr($sign,0,24);
		$this->DB->query("update ".$this->Tbl['user']." set user_signkey='$sign' where user_id=".intval($userid));
		return $sign;
	}



	function writeFile($file,$wtag,$cont)
	{
		$fp=@fopen($file,$wtag);
		@fwrite($fp,$cont);
		@fclose($fp);
	}


	function stripVar(&$val)
	{
		if(is_array($val))
		{
			foreach($val as $key => $v)
			{
				$this->stripVar($val[$key]);
			}
		}
		else
		{
			$val=StripSlashes($val);
		}
	}

	# PHP 1.3
	//@varType: 0 string, 1 int 2 double
	function getpost($fields='',$varType=0)
	{
		$rval=array_merge($_GET,$_POST);
		if(strlen($fields)>0)
		{
			if (isset($rval[$fields]))
			{
				$rval=$rval[$fields];
			}
			else
			{
				$rval='';
			}
			if ($varType>0) $rval=trim($rval);

			if (strlen($rval)>0 and $varType>=1 and $varType<3)
			{
				return ($varType==2 ? doubleval($rval) : intval($rval));
			}
		}
		if (get_magic_quotes_gpc()) $this->stripVar($rval);

		return $rval;
	}

}


?>